#############################################################################
#
# tiktools
#
# TiK super filter package to monitor a chat room for all strings FIRST, and 
# then take some actions based on those strings.
#
#############################################################################
#
# Author: Does not matter
#
# Maintainer: Be my guest, maintain it
#
#############################################################################
#
# Template taken from xlurch.tcl by John "FuzzFace" McMahon (AIM: FuzzFace00)
#
#############################################################################

namespace eval tiktools {

        regexp -- \
        {[0-9]+\.[0-9]+} {@(#)tiktools - $Revision: 1.9 $} \
        ::tiktools::VERSION

        regexp -- { .* } {:$Date: 2000/03/22 22:40:32 $} \
        ::tiktools::VERSDATE

}

namespace eval tiktools {

	variable info

	namespace export load unload goOnline goOffline addKeyword

	proc load {} {
	}

	proc unload {} {
	}

	proc goOnline {} {
	}

	proc goOffline {} {
	}

    proc preproc {xmsg id} {
        set msg [string trimleft [string trimright $xmsg]]
        if {[regexp -nocase -- {(/[a-z0-9]+)} $msg]} {
            if {[regexp -nocase -- {^/rot[0-9]*[ 	]+.+$} $msg]} {
                regsub -nocase -- {^/rot([0-9]*)[ 	]+.*$} $msg {\1} n
                regsub -nocase -- {^/rot[0-9]*[ 	]+(.+)$} $msg {\1} mymsg
                if {[string length $n] == 0} {
                    set msg [rot13 $mymsg]
                } else {
                    set msg [rotn $n $mymsg]
                }
            } elseif {[regexp -nocase -- {^/nick[ 	]+.+$} $msg]} {
                regsub -nocase -- {^/nick[ 	]+(.+)$} $msg {\1} mymsg
                toc_format_nickname $::NSCREENNAME $mymsg
                set msg ""
            } elseif {[regexp -nocase -- {^/ignore[ 	]+.+$} $msg]} {
                if { $id != -1 } {
                    regsub -nocase -- {^/ignore[ 	]+(.+)$} $msg {\1} mymsg
                    p_tik_ignore $id $mymsg
                }
                set msg ""
            } elseif {[regexp -nocase -- {^/ignore[ 	]+.+$} $msg]} {
                set msg ""
            } elseif {[regexp -nocase -- {^/(crazy|eleet|lame)[ 	]+.+$} $msg]} {
                regsub -nocase -- {^/(crazy|eleet|lame)[ 	]+(.+)$} $msg {\2} mymsg
                set msg [lame $mymsg]
            } elseif {[regexp -nocase -- {^/(bw|rev)[ 	]+.+$} $msg]} {
                regsub -nocase -- {^/(bw|rev)[ 	]+(.+)$} $msg {\2} mymsg
                set msg [strrev $mymsg]
            } elseif {[regexp -nocase -- {^/passw(or)?d[ 	]+.+$} $msg]} {
                regsub -nocase -- {^/passwo?r?d[ 	]+(.+)$} $msg {\1} mymsg
                regsub -nocase -- {^([^ 	]+)[ 	]+[^ 	]+$} $mymsg {\1} oldp
                regsub -nocase -- {^[^ 	]+[ 	]+([^ 	]+)$} $mymsg {\1} newp
                toc_change_passwd $::NSCREENNAME $oldp $newp
                set msg ""
            } elseif {[regexp -nocase -- {^/warn[ 	]+.+$} $msg]} {
               regsub -nocase -- {^/warn[ 	]+(.+)$} $msg {\1} mymsg
                if { $id != -1 } {
                    toc_chat_evil $::NSCREENNAME $id [normalize $mymsg] "anon"
                }
                set msg ""
            } elseif {[regexp -nocase -- {^/away([ 	]+.+)$} $msg]} {
                regsub -nocase -- {^/away[ 	]+(.+)$} $msg {\1} awaymsg
                catch {away::set_away $awaymsg}
                set msg ""
            } elseif {[regexp -nocase -- {^/(away|back)$} $msg]} {
                catch {away::back}
                set msg ""
            } elseif {[regexp -nocase -- {^/me([ 	]+.+)?$} $msg]} {
                set msg [string range $msg 4 end]
            } elseif {[regexp -nocase -- {^/(fortune|yow)$} $msg]} {
                set msg [exec $::TIK(options,fortuneprog)]
            } elseif {[regexp -nocase -- {^/(fortune|yow)[ 	]+(.+)$} $msg]} { 
                regsub -nocase -- {^/[^ 	]+[ 	]+([^ 	]+).*$} $msg {\1} mymsg 
                set msg [exec $::TIK(options,fortuneprog) $mymsg] 
            } elseif {[regexp -nocase -- {^/(lookup|dict|webster|word|define)[ 	]+(.+)$} $msg]} {
                regsub -nocase -- {^/[^ 	]+[ 	]+([^ 	]+).*$} $msg {\1} mymsg
                set msg [exec $::TIK(options,websterprog) $mymsg]
            } elseif {[regexp -nocase -- {^/url[ 	]+(.+)$} $msg]} {
                regsub -nocase -- {^/url[ 	]+(.+)$} $msg {\1} mymsg
                regsub -nocase -- {^([^ 	]+).*$} $mymsg {\1} url
                regsub -nocase -- {^[^ 	]+[ 	]+(.*)$} $mymsg {\1} anchor
                if {[string length $anchor] == 0} {
                    set msg "<a href=\"$url\">$url</a>"
                } else {
                    set msg "<a href=\"$url\">$anchor</a>"
                }
                return $msg
            } elseif {[regexp -nocase -- {^/exit$} $msg]} {
                tik_signoff
                exit
            } elseif {[regexp -nocase -- {^/brb$} $msg]} {
                catch {away::set_away "brb"}
                set msg ""
            } elseif {[regexp -nocase -- {^/signoff} $msg]} {
                tik_signoff
                set msg ""
            } elseif {[regexp -nocase  -indices -- {(/(time))} $msg indexes]} {
		puts "Time String"
                append tempmsg [string range $msg 0 [expr ([lindex $indexes 1] - 5)]]
                append tempmsg [clock format [clock seconds] -format "%H:%M %p"]
                append tempmsg [string range $msg [expr ([lindex $indexes 1]+1)] end]
                set msg $tempmsg 
                set tempmsg ""
            }

        }
        if {![regexp -nocase -- {a[ 	]+href} $msg] && ![regexp -nocase -- {mailto:} $msg]} {
            regsub -all -- {([^ 	@]+)(\@[^ 	\@]+)} $msg {<a href="mailto:\1\2">\1\2</a>} xmsg
            regsub -all -- {(http|https|ftp|gopher|news)(:[^ 	]+)} $xmsg {<a href="\1\2">\1\2</a>} msg 
        }
        return $msg
    }

    proc rot13 {str} {
        return [rotn 13 $str]
    }

    proc rotn {n str} {
        if {$n <= 0} {
            return $str
        }
        set abc "abcdefghijklmnopqrstuvwxyz"
        set ABC "ABCDEFGHIJKLMNOPQRSTUVWXYZ"

        set newline ""
        for {set i 0} {$i < [string length "$str"]} {incr i} {
            set char "[string index "$str" $i]"
            set j [string first "$char" "$abc"]
            if {$j != -1} {
                append rotated "[string index "$abc" [expr ($j+$n)%26]]"
                continue
            }
            set j [string first "$char" "$ABC"]
            if {$j != -1} {
                append rotated "[string index "$ABC" [expr ($j+$n)%26]]"
                continue
            }
            append rotated "$char"
        }
        return $rotated
    }

    proc lame {str} {    
        set str [string tolower $str]
        set fromstr "abcdefghijklmnopqrstuvwxyz012358?!"
        set tostr   "48D3FGH1JK1M0Q57VWXZSB"

        set newline ""
        for {set i 0} {$i < [string length "$str"]} {incr i} {
            set char "[string index "$str" $i]"
            set j [string first "$char" "$fromstr"]
            if {$j != -1} {
                append stupified "[string index "$tostr" $j]"
                if {[expr 2 * rand()] > 1} {
                
                }
                continue
            }
            append stupified "$char"
        }
        return $stupified
    }
}

proc strrev {argstr} {
    set ist [split $argstr ""]

    set b ""

    foreach element $ist {
        set b [linsert $b 0 $element]
    }
    return [join $b ""]
}
